<?php
/**
 * Horde Log package
 *
 * This package is based on Zend_Log from the Zend Framework
 * (http://framework.zend.com).  Both that package and this
 * one were written by Mike Naberezny and Chuck Hagenbuch.
 *
 * @category Horde
 * @package  Horde::Log
 * @author   Mike Naberezny (mike@maintainable.com)
 * @author   Chuck Hagenbuch (chuck@horde.org)
 * @license  http://opensource.org/licenses/bsd-license.php BSD
 */

namespace Horde::Log;

import Horde::Log::Exception;
import Horde::Log::Filter_Level;

/**
 * @category Horde
 * @package  Horde::Log
 * @author   Mike Naberezny (mike@maintainable.com)
 * @author   Chuck Hagenbuch (chuck@horde.org)
 * @license  http://opensource.org/licenses/bsd-license.php BSD
 */
class Logger
{
    /**
     * @var array of log levels where the keys are the
     * level priorities and the values are the level names
     */
    private $_levels = array();

    /**
     * @var array of Horde::Log::Handler_Abstract objects
     */
    private $_handlers = array();

    /**
     * @var array of Horde::Log::Filter_Interface objects
     */
    private $_filters = array();

    /**
     * Class constructor.  Create a new logger
     *
     * @param Horde::Log::Handler_Abstract|null  $handler  default handler
     */
    public function __construct($handler = null)
    {
        $r = new ReflectionClass('Horde::Log');
        $this->_levels = array_flip($r->getConstants());

        if ($handler !== null) {
            $this->addHandler($handler);
        }
    }

    /**
     * Undefined method handler allows a shortcut:
     *   $log->levelName('message')
     *     instead of
     *   $log->log('message', Horde::Log::LEVELNAME)
     *
     * @param  string  $method  log level name
     * @param  string  $params  message to log
     * @return void
     */
    public function __call($method, $params)
    {
        $level = strtoupper($method);
        if (($level = array_search($level, $this->_levels)) !== false) {
            $this->log(array_shift($params), $level);
        } else {
            throw new Exception('Bad log level');
        }
    }

    /**
     * Log a message at a level
     *
     * @param  string   $message  Message to log
     * @param  integer  $level    Log level of message
     * @return void
     */
    public function log($message, $level)
    {
        foreach ($this->_filters as $filter) {
            if (!$filter->accept($message, $level)) {
                return;
            }
        }

        if (empty($this->_handlers)) {
            throw new Exception('No handlers were added');
        }

        if (! isset($this->_levels[$level])) {
            throw new Exception('Bad log level');
        }

        foreach ($this->_handlers as $handler) {
            $handler->log($message, $level);
        }
    }

    /**
     * Add a custom log level
     *
     * @param  string  $name    Name of level
     * @param  integer  $level  Numeric level
     * @return void
     */
    public function addLevel($name, $level)
    {
        // Log level names must be uppercase for predictability.
        $name = strtoupper($name);

        if (isset($this->_levels[$level])
            || array_search($name, $this->_levels)) {
            throw new Exception('Existing log levels cannot be overwritten');
        }

        $this->_levels[$level] = $name;
    }

    /**
     * Add a filter that will be applied before all log handlers.
     * Before a message will be received by any of the handlers, it
     * must be accepted by all filters added with this method.
     *
     * @param  Horde::Log::Filter_Interface  $filter
     * @return void
     */
    public function addFilter($filter)
    {
        if (is_integer($filter)) {
            $filter = new Filter_Level($filter);
        }

        $this->_filters[] = $filter;
    }

    /**
     * Add a handler.  A handler is responsible for taking a log
     * message and writing it out to storage.
     *
     * @param  Horde::Log::Handler_Abstract $handler
     * @return void
     */
    public function addHandler($handler)
    {
        $this->_handlers[] = $handler;
    }

}
